package com.example.parkreminder;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;


import android.Manifest;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.Priority;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.Task;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.CompletableFuture;

import render.animations.Flip;
import render.animations.Render;

public class MainActivity extends AppCompatActivity {

    public static final int REQUEST_CODE = 100;
    private RelativeLayout RLFindLocation, RL_RLCurrentLocation, RL_GetDirectionBtn, RL_GetDirection;

    private TextView tvCurrentLocation, tv_lastLocation;

    private Database database;

    private FusedLocationProviderClient fusedLocationProviderClient;

    private String street, lon, lat;

    private Render render = new Render(MainActivity.this);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        tv_lastLocation = findViewById(R.id.tv_lastLocation);
        tvCurrentLocation = findViewById(R.id.tvCurrentLocation);
        RLFindLocation = findViewById(R.id.RLFindLocation);
        RL_RLCurrentLocation = findViewById(R.id.RL_RLCurrentLocation);
        RL_GetDirectionBtn = findViewById(R.id.RL_GetDirectionBtn);
        RL_GetDirection = findViewById(R.id.RL_GetDirection);

        RLFindLocation.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                findLocation();
            }
        });

        RL_GetDirectionBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                openGoogleMaps();
            }
        });

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this);
        askPermission();

        database = new Database(this);

        fetchLastLocation();
    }

    private void findLocation() {
        if(ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) return;

        Task<Location> locationTask =
                fusedLocationProviderClient.getCurrentLocation(Priority.PRIORITY_HIGH_ACCURACY, null);

        locationTask.addOnCompleteListener(new OnCompleteListener<Location>() {
            @Override
            public void onComplete(@NonNull Task<Location> task) {
                Location location = task.getResult();

                // Geocoder needs Internet, This operation may delay, use separate thread!
                CompletableFuture
                    .supplyAsync(() -> getAddresses(location))
                    .thenAccept(list -> {
                        if(list != null && !list.isEmpty()) {
                            String street = list.get(0).getAddressLine(0);
                            double lon = list.get(0).getLongitude();
                            double lat = list.get(0).getLatitude();

                            Log.d("Addresses", "street: " + street + " lon: " + lon + "lat: " + lat);

                            saveLocation(street, lon, lat);
                        }else{
                            // Geocoder not available, use lon, lat from location task, For Street name,
                            // lets named it something like "Car Location"
                            saveLocation("Car Location", location.getLongitude(), location.getLongitude());
                        }
                    });
            }
        });

        locationTask.addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception e) {
                Log.e("ERROR", ""+e);
                // TODO: Handle Failures, Inform User
            }
        });
    }

    private void saveLocation(String street, double lon, double lat) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm");
        Date date = new Date(System.currentTimeMillis());

        String timeStamp = simpleDateFormat.format(date);

        long id = database.insert(street, lon, lat, timeStamp);

        if(id == -1){
            Log.e("ERROR INSERT TO DB", "id: "+id);
            // TODO: Handle Failures, Inform User
            return;
        }

        setNotification(street);

        // Only main thread can update UI, This method runs on separate thread, use runOnUiThread(..)
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                tvCurrentLocation.setText(street);
                RL_GetDirection.setVisibility(View.INVISIBLE);
                RL_RLCurrentLocation.setVisibility(View.VISIBLE);
                render.setAnimation(Flip.InX(RL_RLCurrentLocation));
                render.start();
            }
        });
    }

    private List<Address> getAddresses(Location location){
        Geocoder geocoder = new Geocoder(this, Locale.getDefault());
        try {
            List<Address> addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
            return addresses;
        }catch (Exception e){
            return null;
        }
    }

    private void setNotification(String street){
        Calendar calendar = Calendar.getInstance();
        // Modify for your needs! You can use calendar.add(Calendar.HOUR) etc
        calendar.add(Calendar.MINUTE, 1);

        Intent intent = new Intent(this, AlertReceiver.class);
        intent.putExtra("street", street);
        // Use putExtra for more data to pass to the AlertReceiver

        PendingIntent pendingIntent = PendingIntent.getBroadcast(this, REQUEST_CODE, intent,
                PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);

        AlarmManager alarmManager = (AlarmManager) getSystemService(ALARM_SERVICE);
        // If you want to repeat, use alarmManager.setRepeating(...);
        alarmManager.setExact(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), pendingIntent);
    }

    private void openGoogleMaps(){
        if(lat == null || lon == null || street == null) return;

        Log.d("OPEN GOOGLE MAPS", ""+ lat + "," + lon + " (" + street);
        String geoUri = "http://maps.google.com/maps?q=loc:" + lat + "," + lon + " (" + street + ")";
        Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(geoUri));
        startActivity(intent);
    }

    private void fetchLastLocation(){
        // Fetching data from DB may blocking main thread. Consider using separate thread!
        // TODO: Use CompletableFuture.supplyAsync(..)

        Cursor cursor = database.fetchLast();

        if(cursor == null || cursor.getCount() <= 0) return;

        while (cursor.moveToNext()){
            street = cursor.getString(1);
            lon = cursor.getString(2);
            lat = cursor.getString(3);
            String timestamp = cursor.getString(4);
        }

        tv_lastLocation.setText(street);
        RL_GetDirection.setVisibility(View.VISIBLE);
    }

    public static final int LOC_CODE = 44;

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if(requestCode == LOC_CODE && grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            // TODO: Handle Permission Granted
        }else{
            // TODO: Handle Permission Denied
        }
    }

    public void askPermission(){
        if(ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(this, new String[]{ Manifest.permission.ACCESS_FINE_LOCATION }, LOC_CODE);
        }
    }
}